/*

  Three Rotating Triangles Demo
  By David Springate <djspringate@mail.ru>

  With thanks to Morten Mikkelsen and Steven Osman

  9.1.04 01:38 Version 0.5

  LATEST: Changed rotation function to have the same inputs as OpenGL's glRotate.

*/

#include <stdio.h>
#include <math.h>

#include <sps2lib.h>
#include <sps2tags.h>
#include <sps2util.h>

typedef int vertex_t[3];
typedef int color_t[3];

int movingAngle;

void drawTriangle();
void triRotate(vertex_t *vertices, float angle, int switchX, int switchY, int switchZ);
void constructGIF(vertex_t *vertices, color_t *colors);

int main() {
  int iSPS2Descriptor;

  iSPS2Descriptor=sps2Init();

  sps2UScreenInit(0);

  movingAngle = 0;

  while (1) {
    sps2UScreenClear(0,0,0);

    drawTriangle();

    movingAngle++;

    if (movingAngle > 360)
      movingAngle = 0;

    sps2UScreenSwap();
  }
  
  sps2UScreenShutdown();

  sps2Release(iSPS2Descriptor);
  return 0;
}

void triRotate(vertex_t *vertices, float angle, int switchX, int switchY, int switchZ){
  double pi = 3.14159265;
  double radian;

  int i = 0;

  int x;
  int y;
  int z;

  double cosAngle, sinAngle;
  int a,b,c;

  radian = angle * (pi/180);

  cosAngle = cos(radian);
  sinAngle = sin(radian);

  while (i < 3){
    x = vertices[i][0];
    y = vertices[i][1];
    z = vertices[i][2];

    a=b=c=0;
    
    if (switchX == 1){
      a+=x;
      b+=(cosAngle*y)-(sinAngle*z);
      c+=(sinAngle*y)+(cosAngle*z);
    }

    if (switchY == 1){
      a+=(cosAngle*x)+(sinAngle*z);
      b+=y;
      c+=(-sinAngle*x)+(cosAngle*z);
    }

    if (switchZ == 1){
      a+=(cosAngle*x)-(sinAngle*y);
      b+=(sinAngle*x)+(cosAngle*y);
      c+=z;
    }

    vertices[i][0]=a;
    vertices[i][1]=b;
    vertices[i][2]=c;
    i++;
  }

}

void drawTriangle() {

  vertex_t aVertices[]={
    {  0,  -25, 0},
    {  -25,  25, 0 },
    { 25, 25, 0}
  };

  vertex_t bVertices[]={
    { -75, -75, 0},
    { -100, -25, 0},
    { -50, -25, 0}
  };

  vertex_t cVertices[]={
    { 75, -75, 0},
    { 50, -25, 0},
    {100, -25, 0},
  };

  color_t aColors[]={
    {255,  0,  0},
    {  0,255,  0},
    {  0,  0,255}
  };
  
  triRotate(aVertices, movingAngle, 0,1,0);
  constructGIF(aVertices, aColors);

  triRotate(bVertices, movingAngle, 1,0,0);
  constructGIF(bVertices, aColors);

  triRotate(cVertices, movingAngle, 0,0,1);
  constructGIF(cVertices, aColors);
}

void constructGIF(vertex_t *vertices, color_t *colors){
  sps2GIFTag_t gifTag;
  sps2GIFPackedRegister_t gifRegister;
  int iPointLooper;

  gifTag.i128=0;       // Blank it out
  gifTag.s.NLOOP=3;    // 3 entries.  One entry for each of the points.
  gifTag.s.PRE=1;      // We are providing a valid PRIM value
  gifTag.s.EOP=1;      // End of packet
  gifTag.s.PRIM=11;    // 3 = triangle | 8 = gouraud shading | 64 = alpha blending
  gifTag.s.FLG=GIF_FLG_PACKED; // We're using the packed format for data
  gifTag.s.NREG=0;     // Start off with zero registers, increment as we populate the descriptions
  
  SPS2_SET_GIF_REG(gifTag, gifTag.s.NREG++, GIF_REG_RGBAQ);
  SPS2_SET_GIF_REG(gifTag, gifTag.s.NREG++, GIF_REG_XYZ2);

  DPUT_EE_GIF_FIFO(gifTag.i128);

  for (iPointLooper=0; iPointLooper<3;iPointLooper++) {
    gifRegister.i128=0;
    gifRegister.RGBAQ.s.R=colors[iPointLooper][0];
    gifRegister.RGBAQ.s.G=colors[iPointLooper][1];
    gifRegister.RGBAQ.s.B=colors[iPointLooper][2];
    DPUT_EE_GIF_FIFO(gifRegister.i128);

    gifRegister.i128=0;
    gifRegister.XYZ2.s.X=(vertices[iPointLooper][0]+2048) << 4;
    gifRegister.XYZ2.s.Y=(vertices[iPointLooper][1]+2048) << 4;
    gifRegister.XYZ2.s.Z=(vertices[iPointLooper][2]+2048) << 4;
    DPUT_EE_GIF_FIFO(gifRegister.i128);
  }
}
















